package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.BaseDieselGeneratorEnt;
import pps.core.base.entity.BasePowerLineDieselEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BaseDieselGeneratorMapper;
import pps.core.base.mapper.BasePowerLineDieselMapper;
import pps.core.base.service.data.base_diesel_generator.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Objects;

/**
 * 柴发设备模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseDieselGeneratorService {

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseDieselGenerator(XContext context, CreateBaseDieselGeneratorInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
            BaseDieselGeneratorEnt entity = XCopyUtils.copyNewObject(input, BaseDieselGeneratorEnt.class);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseDieselGenerator(XContext context, UpdateBaseDieselGeneratorInput input) {
        return XTransactionHelper.begin(context, () -> {
            BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
            BaseDieselGeneratorEnt entity = this.selectOneByDieselId(input.getId(), mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseDieselGenerator(XContext context, DeleteBaseDieselGeneratorInput input) {
        String dieselId = input.getId();
        if (this.checkDieselIsReference(context, dieselId)) {
            return XServiceResult.error(context, BusinessError.DieselReference);
        }
        return XTransactionHelper.begin(context, () -> {
            BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
            BaseDieselGeneratorEnt entity = this.selectOneByDieselId(dieselId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            entity = new BaseDieselGeneratorEnt();
            entity.setId(dieselId);
            entity.setIsDeleted(BusinessConstant.ZERO);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("根据id获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseDieselGeneratorOutput> getBaseDieselGenerator(XContext context, GetBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        BaseDieselGeneratorEnt entity = this.selectOneByDieselId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDieselGeneratorOutput output = XCopyUtils.copyNewObject(entity, GetBaseDieselGeneratorOutput.class);
        return XSingleResult.success(output);
    }

    /**
     * 分页查询
     * GET /base/base-diesel-generator/query-base-diesel-generator
     * 接口ID：105601992
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105601992
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBaseDieselGeneratorOutput}>
     */
    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDieselGeneratorOutput> queryBaseDieselGenerator(XContext context, QueryBaseDieselGeneratorInput input) {
        String ouName = input.getOuName();
        String deviceName = input.getDeviceName();
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        if (StringUtils.equals(BusinessConstant.STRING_ONE, input.getExcludeIdFlag())) {
            queryWrapper.lambda()
                    .notExists("SELECT 1 FROM base_power_line_diesel w WHERE base_diesel_generator.id = w.diesel_id AND w.is_deleted = 1")
                    .eq(BaseDieselGeneratorEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BaseDieselGeneratorEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .like(StrUtil.isNotEmpty(ouName), BaseDieselGeneratorEnt::getOuName, ouName)
                .like(StrUtil.isNotEmpty(deviceName), BaseDieselGeneratorEnt::getDeviceName, deviceName)
                .eq(BaseDieselGeneratorEnt::getIsDeleted, BusinessConstant.ONE)
                .orderByDesc(BaseDieselGeneratorEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseDieselGeneratorOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查柴发设备是否被引用
     *
     * @param context  上下文
     * @param dieselId 柴发设备ID
     * @return boolean
     */
    private boolean checkDieselIsReference(XContext context, String dieselId) {
        BasePowerLineDieselMapper mapper = context.getBean(BasePowerLineDieselMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineDieselEnt>()
                .eq(BasePowerLineDieselEnt::getDieselId, dieselId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 通过设备ID查询设备信息
     *
     * @param dieselId 设备id
     * @param mapper   映射器
     * @return {@link BaseDieselGeneratorEnt}
     */
    private BaseDieselGeneratorEnt selectOneByDieselId(String dieselId, BaseDieselGeneratorMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BaseDieselGeneratorEnt>()
                .eq(BaseModel::getId, dieselId)
        );
    }
}