package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.BasePhotovoltaicPlantEnt;
import pps.core.base.entity.BasePowerLineEnt;
import pps.core.base.entity.BasePowerLinePlantEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BasePhotovoltaicPlantMapper;
import pps.core.base.mapper.BasePowerLinePlantMapper;
import pps.core.base.service.data.base_photovoltaic_plant.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.constant.UserConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.system.constant.SysOrganizationCode;
import pps.core.system.entity.SysOrganizationView;
import pps.core.system.mapper.SysOrganizationViewMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Objects;


/**
 * 光伏电站模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BasePhotovoltaicPlantService {

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBasePhotovoltaicPlant(XContext context, CreateBasePhotovoltaicPlantInput input) {
        return XTransactionHelper.begin(context, () -> {
            BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
            BasePhotovoltaicPlantEnt entity = XCopyUtils.copyNewObject(input, BasePhotovoltaicPlantEnt.class);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBasePhotovoltaicPlant(XContext context, UpdateBasePhotovoltaicPlantInput input) {
        return XTransactionHelper.begin(context, () -> {
            BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
            BasePhotovoltaicPlantEnt entity = this.selectOneByPlantId(input.getId(), mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBasePhotovoltaicPlant(XContext context, DeleteBasePhotovoltaicPlantInput input) {
        String plantId = input.getId();
        if (this.checkPhotovoltaicIsReference(context, plantId)) {
            return XServiceResult.error(context, BusinessError.PhotovoltaicReference);
        }
        return XTransactionHelper.begin(context, () -> {
            BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
            BasePhotovoltaicPlantEnt entity = this.selectOneByPlantId(plantId, mapper);
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            entity = new BasePhotovoltaicPlantEnt();
            entity.setId(plantId);
            entity.setIsDeleted(BusinessConstant.ZERO);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XText("根据id获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePhotovoltaicPlantOutput> getBasePhotovoltaicPlant(XContext context, GetBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        BasePhotovoltaicPlantEnt entity = this.selectOneByPlantId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePhotovoltaicPlantOutput output = XCopyUtils.copyNewObject(entity, GetBasePhotovoltaicPlantOutput.class);
        //根据组织机构的路径获取
        SysOrganizationViewMapper organizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView organizationView = new SysOrganizationView();
        organizationView.setId(entity.getOuId());
        organizationView = organizationViewMapper.selectOrgProvince(organizationView);
        if (Objects.nonNull(organizationView)) {
            output.setProvince(organizationView.getProvinceName());
        }
        return XSingleResult.success(output);
    }

    /**
     * 分页查询
     * GET /base/base-photovoltaic-plant/query-base-photovoltaic-plant
     * 接口ID：105466277
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466277
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePhotovoltaicPlantOutput}>
     */
    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePhotovoltaicPlantOutput> queryBasePhotovoltaicPlant(XContext context, QueryBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        String stationName = input.getStationName();
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        if (StringUtils.equals(BusinessConstant.STRING_ONE, input.getExcludeIdFlag())) {
            queryWrapper.lambda()
                    .notExists("SELECT 1 FROM base_power_line_plant w WHERE base_photovoltaic_plant.id = w.plant_id AND w.is_deleted = 1")
                    .eq(BasePhotovoltaicPlantEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BasePhotovoltaicPlantEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .like(StrUtil.isNotEmpty(stationName), BasePhotovoltaicPlantEnt::getStationName, stationName)
                .eq(BasePhotovoltaicPlantEnt::getIsDeleted, BusinessConstant.ONE)
                .orderByDesc(BasePhotovoltaicPlantEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePhotovoltaicPlantOutput.class);
    }

    @XText("根据父级id查询儿子级(含电站)")
    @XApiGet
    public XListResult<GetSysOrganizationAndPlantViewOutput> selectOuAndPhotovoltaicPlantInfoByParentOuId(XContext context, GetSysOrganizationAndPlantInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        BasePhotovoltaicPlantMapper plantMapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(SysOrganizationCode.NORMAL);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        List<SysOrganizationView> viewList;
        String ouId = "00000000-0000-0000-0000-000000000000";
        if (XStringUtils.isEmpty(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (UserConstant.ISSUPERADMIN_TRUE.equals(isSuperAdmin)) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                ouId = session.getOuId();
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            ouId = input.getParentOuId();
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        queryWrapper.lambda().eq(BasePhotovoltaicPlantEnt::getOuId, ouId).eq(BasePhotovoltaicPlantEnt::getIsDeleted, BusinessConstant.ONE);
        List<BasePhotovoltaicPlantEnt> plantList = plantMapper.selectList(queryWrapper);
        plantList.forEach(item -> {
            SysOrganizationView view = new SysOrganizationView();
            view.setId(item.getId());
            view.setOuName(item.getStationName());
            view.setShortName(item.getStationName());
            view.setOuLevel("PHOTOVOLTAIC");
            viewList.add(view);
        });
        List<GetSysOrganizationAndPlantViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationAndPlantViewOutput.class);
        return XListResult.success(outputs);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查光伏电站是否被引用
     *
     * @param context 上下文
     * @param plantId 电站ID
     * @return boolean
     */
    private boolean checkPhotovoltaicIsReference(XContext context, String plantId) {
        BasePowerLinePlantMapper mapper = context.getBean(BasePowerLinePlantMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLinePlantEnt>()
                .eq(BasePowerLinePlantEnt::getPlantId, plantId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 通过电站ID获取电站信息
     *
     * @param plantId 电站id
     * @param mapper  映射器
     * @return {@link BasePowerLineEnt}
     */
    private BasePhotovoltaicPlantEnt selectOneByPlantId(String plantId, BasePhotovoltaicPlantMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                .eq(BaseModel::getId, plantId));
    }
}
