package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceCalibrationHistoryView;
import pps.core.space.entity.SpaceCalibrationPeriodEnt;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceInstitutionWellheadEnt;
import pps.core.space.enums.BusinessError;
import pps.core.space.mapper.SpaceCalibrationHistoryViewMapper;
import pps.core.space.mapper.SpaceCalibrationPeriodMapper;
import pps.core.space.mapper.SpaceInstitutionDetailMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadMapper;
import pps.core.space.service.data.space_calibration_history.QuerySpaceCalibrationHistoryInput;
import pps.core.space.service.data.space_calibration_history.QuerySpaceCalibrationHistoryOutput;
import pps.core.space.service.data.space_calibration_history.QuerySpaceCalibrationHistoryViewInput;
import pps.core.space.service.data.space_calibration_history.QuerySpaceCalibrationHistoryViewOutput;
import pps.core.space.service.data.space_calibration_period.CreateSpaceCalibrationPeriodInput;
import pps.core.space.service.data.space_calibration_period.GetSpaceCalibrationPeriodViewInput;
import pps.core.space.service.data.space_calibration_period.GetSpaceCalibrationPeriodViewOutput;
import pps.core.space.service.data.space_calibration_period.UpdateSpaceCalibrationPeriodInput;
import pps.core.space.service.data.space_institution_wellhead.QuerySpaceInstitutionWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 定期校准模块
 *
 * @author ZWT
 * @date 2023/09/06
 */
@XService
@XText("定期校准模块")
public class SpaceCalibrationService {

    /**
     * 校准周期--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("校准周期--新增")
    public XServiceResult createSpaceCalibrationPeriod(XContext context, CreateSpaceCalibrationPeriodInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
        SpaceCalibrationPeriodEnt periodEnt = this.getPeriodEntByParam(mapper, input.getLineId(), input.getOuId());
        if (Objects.nonNull(periodEnt)) {
            return XServiceResult.error(context, BusinessError.LineExists);
        }
        SpaceCalibrationPeriodEnt entity = XCopyUtils.copyNewObject(input, SpaceCalibrationPeriodEnt.class);
        //查生效中的基础间开
        String lineId = entity.getLineId();
        //取当前时间
        DateTime date = DateUtil.date();
        String sameDay = date.toString(BusinessConstant.DATE_FORMAT_DAY);
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        List<SpaceInstitutionDetailEnt> detailEntList = detailMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                .eq(SpaceInstitutionDetailEnt::getLineId, lineId)
                .le(SpaceInstitutionDetailEnt::getInstitutionStartDate, sameDay)
                .ge(SpaceInstitutionDetailEnt::getInstitutionEndDate, sameDay)
        );
        List<SpaceCalibrationHistoryView> historyViewList = new ArrayList<>(8);
        if (CollUtil.isNotEmpty(detailEntList)) {
            for (SpaceInstitutionDetailEnt detailEnt : detailEntList) {
                //创建周期
                this.createCalibrationHistory(historyViewList, lineId, detailEnt.getId(), date, Integer.valueOf(entity.getDayNumber()));
            }
        }
        return XTransactionHelper.begin(context, () -> {
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            SpaceCalibrationHistoryViewMapper historyViewMapper = context.getBean(SpaceCalibrationHistoryViewMapper.class);
            if (CollUtil.isNotEmpty(historyViewList)) {
                historyViewMapper.batchInsertList(historyViewList);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 校准周期--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("校准周期--修改")
    public XServiceResult updateSpaceCalibrationPeriod(XContext context, UpdateSpaceCalibrationPeriodInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
            SpaceCalibrationPeriodEnt entity = mapper.selectOne(
                    new LambdaQueryWrapper<SpaceCalibrationPeriodEnt>()
                            .eq(BaseModel::getId, input.getId())
            );
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 校准周期--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceCalibrationPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("校准周期--详情")
    public XSingleResult<GetSpaceCalibrationPeriodViewOutput> getSpaceCalibrationPeriodView(XContext context, GetSpaceCalibrationPeriodViewInput input) {
        String lineId = input.getLineId();
        String ouId = input.getOuId();
        SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
        SpaceCalibrationPeriodEnt entity = this.getPeriodEntByParam(mapper, lineId, ouId);
        GetSpaceCalibrationPeriodViewOutput output = new GetSpaceCalibrationPeriodViewOutput();
        if (Objects.nonNull(entity)) {
            XCopyUtils.copyObject(entity, output);
        }
        output.setLineName(
                ServiceUtil.getPowerLineNameById(context, lineId)
        );
        output.setOuName(
                ServiceUtil.getSysOrgNameById(context, ouId)
        );
        return XSingleResult.success(output);
    }

    /**
     * 校准历史--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceCalibrationHistoryViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("校准历史--分页列表")
    public XPageResult<QuerySpaceCalibrationHistoryViewOutput> querySpaceCalibrationHistoryView(XContext context, QuerySpaceCalibrationHistoryViewInput input) {
        SpaceCalibrationHistoryViewMapper mapper = context.getBean(SpaceCalibrationHistoryViewMapper.class);
        SpaceCalibrationHistoryView record = XCopyUtils.copyNewObject(input, SpaceCalibrationHistoryView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceCalibrationHistoryView> list = mapper.selectList(record);
        PageInfo<SpaceCalibrationHistoryView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceCalibrationHistoryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceCalibrationHistoryViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceCalibrationHistoryViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 校准周期--获取线路及井口树
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link QuerySpaceCalibrationHistoryOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("校准周期--获取线路及井口树")
    public XSingleResult<QuerySpaceCalibrationHistoryOutput> querySpaceCalibrationLineView(XContext context, QuerySpaceCalibrationHistoryInput input) {
        QuerySpaceCalibrationHistoryOutput output = new QuerySpaceCalibrationHistoryOutput();
        String institutionId = input.getInstitutionId();
        String lineId = input.getLineId();
        SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        //查井口列表
        List<SpaceInstitutionWellheadEnt> wellheadEntList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                .eq(SpaceInstitutionWellheadEnt::getInstitutionId, institutionId)
                .orderByAsc(SpaceInstitutionWellheadEnt::getStartSeq)
        );
        if (CollUtil.isNotEmpty(wellheadEntList)) {
            output.setWellheadList(XCopyUtils.copyNewList(wellheadEntList, QuerySpaceInstitutionWellheadOutput.class));
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        output.setInstitutionId(institutionId);
        output.setLineId(lineId);
        output.setLineName(ServiceUtil.getPowerLineNameById(context, lineId));
        return XSingleResult.success(output);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 通过参数获取周期
     *
     * @param mapper 映射器
     * @param lineId 线路id
     * @param ouId   ou-id
     * @return {@link SpaceCalibrationPeriodEnt}
     */
    private SpaceCalibrationPeriodEnt getPeriodEntByParam(SpaceCalibrationPeriodMapper mapper, String lineId, String ouId) {
        return mapper.selectOne(
                new LambdaQueryWrapper<SpaceCalibrationPeriodEnt>()
                        .eq(SpaceCalibrationPeriodEnt::getLineId, lineId)
                        .eq(SpaceCalibrationPeriodEnt::getOuId, ouId)
        );
    }

    /**
     * 创建校准历史记录
     *
     * @param historyViewList 历史视图列表
     * @param lineId          线路id
     * @param institutionId   机构id
     * @param date            日期
     * @param dayNumber       天数
     */
    private void createCalibrationHistory(List<SpaceCalibrationHistoryView> historyViewList, String lineId,
                                          String institutionId, Date date, Integer dayNumber) {
        SpaceCalibrationHistoryView historyView = new SpaceCalibrationHistoryView();
        BaseUtils.setBaseModelDefaultForJob(historyView);
        historyView.setLineId(lineId);
        historyView.setInstitutionId(institutionId);
        historyView.setExecutionCycle(BaseUtils.getExecutionCycleForCalibration(date, dayNumber));
        historyView.setCalibrationDate(DateUtil.beginOfDay(DateUtil.offsetDay(date, dayNumber + 1)));
        historyViewList.add(historyView);
    }

}
