package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_long_period.GetSpaceOptimizeLongPeriodViewInput;
import pps.core.space.service.data.space_optimize_long_period.GetSpaceOptimizeLongPeriodViewOutput;
import pps.core.space.service.data.space_optimize_long_period.QuerySpaceOptimizeLongPeriodViewInput;
import pps.core.space.service.data.space_optimize_long_period.QuerySpaceOptimizeLongPeriodViewOutput;
import pps.core.space.service.data.space_optimize_long_wellhead.GetSpaceOptimizeLongWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 长期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("长期间开优化模块")
public class SpaceOptimizeLongPeriodService {

    /**
     * 长期间开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeLongPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("长期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeLongPeriodViewOutput> querySpaceOptimizeLongPeriodView(XContext context, QuerySpaceOptimizeLongPeriodViewInput input) {
        SpaceOptimizeLongPeriodViewMapper mapper = context.getBean(SpaceOptimizeLongPeriodViewMapper.class);
        SpaceOptimizeLongPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeLongPeriodView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeLongPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeLongPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeLongPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeLongPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeLongPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? BusinessConstant.OPTIMIZED_C : BusinessConstant.NO_OPTIMIZATION_C);
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 长期间开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizeLongPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("长期间开优化--详情")
    public XSingleResult<GetSpaceOptimizeLongPeriodViewOutput> getSpaceOptimizeLongPeriodView(XContext context, GetSpaceOptimizeLongPeriodViewInput input) {
        SpaceOptimizeLongPeriodMapper mapper = context.getBean(SpaceOptimizeLongPeriodMapper.class);
        String longPeriodId = input.getId();
        SpaceOptimizeLongPeriodEnt period = mapper.selectById(longPeriodId);
        if (Objects.isNull(period)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceOptimizeLongPeriodViewOutput output = XCopyUtils.copyNewObject(period, GetSpaceOptimizeLongPeriodViewOutput.class);
        output.setDateList(ServiceUtil.getOptimizeDateList(period.getCreateTime(), period.getOptimizeDeadline()));
        //查间开制度详情
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionDetailEnt detailEnt = detailMapper.selectById(period.getInstitutionId());
        if (StringUtils.equals(String.valueOf(BusinessConstant.ONE), Optional.ofNullable(detailEnt.getGridTypeKey())
                .orElse(String.valueOf(BusinessConstant.ZERO)))) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(period.getOptimizeDeadline());
            output.setStrategyDetailList(ServiceUtil.getStrategyDetailList(context,
                    GetBasePriceStrategyDetailInput.builder()
                            .lineId(period.getLineId())
                            .strategyMonth(String.valueOf(calendar.get(Calendar.MONTH) + 1))
                            .build()));
        }
        output.setGridTypeKey(detailEnt.getGridTypeKey());
        //查井口列表
        SpaceOptimizeLongWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeLongWellheadMapper.class);
        List<SpaceOptimizeLongWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeLongWellheadEnt>()
                .eq(SpaceOptimizeLongWellheadEnt::getLongPeriodId, longPeriodId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        if (CollUtil.isNotEmpty(wellheadList)) {
            List<GetSpaceOptimizeLongWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeLongWellheadOutput.class);
            //查井口配置列表
            SpaceOptimizeLongDurationMapper durationMapper = context.getBean(SpaceOptimizeLongDurationMapper.class);
            List<SpaceOptimizeLongDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeLongDurationEnt>()
                    .eq(SpaceOptimizeLongDurationEnt::getLongPeriodId, longPeriodId)
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .orderByAsc(SpaceOptimizeLongDurationEnt::getIsOptimize, SpaceOptimizeLongDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isNotEmpty(durationList)) {
                List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
                Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                        .collect(
                                Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId)
                        );
                wellheadOutputs.forEach(w -> {
                    if (collect.containsKey(w.getWellheadId())) {
                        Map<Integer, List<SpaceOptimizeDateDuration>> listMap = collect.get(w.getWellheadId()).stream()
                                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getIsOptimize));
                        w.setOptimizeList(ServiceUtil.getAllDateDuration(listMap.get(0)));
                        w.setInitializeList(ServiceUtil.getAllDateDuration(listMap.get(1)));
                    } else {
                        w.setOptimizeList(new ArrayList<>(0));
                        w.setInitializeList(new ArrayList<>(0));
                    }
                });
            }
            output.setWellheadList(wellheadOutputs);
        } else {
            output.setWellheadList(new ArrayList<>(0));
        }
        return XSingleResult.success(output);
    }
}