package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 短期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/20 11:44
 */
@XService
public class SpaceOptimizeShortCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeShortCloudService {

    /**
     * 短期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeShortJob(XContext context) {
        //取生效中的基础间开
        DateTime date = DateUtil.date();
        String curDate = date.toString(BusinessConstant.DATE_FORMAT_DAY);
        List<SpaceInstitutionDetailEnt> detailEntList = super.getEffectiveSpaceInstitution(context, curDate);
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        DateTime beginOfDay = DateUtil.beginOfDay(date);
        //优化
        for (SpaceInstitutionDetailEnt detail : detailEntList) {
            String detailId = detail.getId();
            String lineId = detail.getLineId();
            //查是否已经存有短期历史数据
            SpaceOptimizeShortPeriodMapper shortPeriodMapper = context.getBean(SpaceOptimizeShortPeriodMapper.class);
            SpaceOptimizeShortPeriodEnt shortPeriod = shortPeriodMapper.selectOne(new LambdaQueryWrapper<SpaceOptimizeShortPeriodEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ZERO)
                    .eq(SpaceOptimizeShortPeriodEnt::getOptimizeDate, beginOfDay)
                    .eq(SpaceOptimizeShortPeriodEnt::getInstitutionId, detailId)
            );
            String periodId;
            if (Objects.isNull(shortPeriod)) {
                //创建记录
                periodId = super.createOptimizePeriod(periodDTOList, detailId, lineId, null, beginOfDay);
            } else {
                periodId = shortPeriod.getId();
            }
            //查未记录的井口
            SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
            List<SpaceInstitutionWellheadEnt> wellheadEntList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceInstitutionWellheadEnt::getInstitutionId, detailId)
                    .notExists("SELECT 1 FROM space_optimize_short_wellhead WHERE is_deleted = 1 AND short_period_id = {0}", periodId)
            );
            if (CollUtil.isEmpty(wellheadEntList)) {
                continue;
            }
            List<String> wellheadIdList = wellheadEntList.stream()
                    .map(SpaceInstitutionWellheadEnt::getWellheadId)
                    .collect(Collectors.toList());
            //查需要初始化的时段配置
            SpaceInstitutionDurationMapper durationMapper = context.getBean(SpaceInstitutionDurationMapper.class);
            List<SpaceInstitutionDurationEnt> durationEntList = durationMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionDurationEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceInstitutionDurationEnt::getInstitutionId, detailId)
                    .in(SpaceInstitutionDurationEnt::getWellheadId, wellheadIdList)
            );
            if (CollUtil.isEmpty(durationEntList)) {
                continue;
            }
            Map<String, List<SpaceInstitutionDurationEnt>> durationListMap = durationEntList.stream()
                    .collect(Collectors.groupingBy(SpaceInstitutionDurationEnt::getWellheadId));
            //按类型拆分井口
            Map<Boolean, List<SpaceInstitutionWellheadEnt>> interval = wellheadEntList.stream()
                    .sorted(Comparator.comparing(SpaceInstitutionWellheadEnt::getStartSeq))
                    .collect(
                            Collectors.partitioningBy(w -> StringUtils.equals(detailId, w.getInstitutionId()) &&
                                    StringUtils.equals(BusinessConstant.INTERVAL_PUMPING_WELL, w.getRunTypeKey()) &&
                                    StringUtils.equals("0", w.getIntervalTypeKey())
                            )
                    );
            //不需要优化的井口
            List<SpaceInstitutionWellheadEnt> unOptimizeWellheadList = interval.get(false);
            if (CollUtil.isNotEmpty(unOptimizeWellheadList)) {
                for (SpaceInstitutionWellheadEnt wellhead : unOptimizeWellheadList) {
                    String wellheadId = wellhead.getWellheadId();
                    String recordId = super.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber());
                    switch (wellhead.getIntervalTypeKey()) {
                        //小间开(不优化)
                        case "0":
                            if (!durationListMap.containsKey(wellheadId)) {
                                continue;
                            }
                            //初始化未优化间开
                            List<SpaceInstitutionDurationEnt> durationList = durationListMap.get(wellheadId);
                            for (SpaceInstitutionDurationEnt durationEnt : durationList) {
                                super.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId);
                                super.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                        durationEnt.getOpenWellTime(), durationEnt.getCloseWellTime()
                                );
                            }
                            break;
                        //大间开
                        case "1":
                            boolean openFlag = true;
                            //计算开关井周期总天数
                            Integer openWellDay = Objects.isNull(wellhead.getOpenWellDay()) ? 0 : wellhead.getOpenWellDay();
                            Integer closeWellDay = Objects.isNull(wellhead.getCloseWellDay()) ? 0 : wellhead.getCloseWellDay();
                            Integer totalDays = openWellDay + closeWellDay;
                            if (!totalDays.equals(0)) {
                                //获取间开井口创建时间和当天的时间差
                                int betweenDay = (int) DateUtil.betweenDay(wellhead.getCreateTime(), date, true);
                                switch (totalDays.compareTo(betweenDay)) {
                                    case -1:
                                        //按周期取余数和开井时间比较,超过开井时间就是关井
                                        if (openWellDay.compareTo(betweenDay % totalDays) >= 0) {
                                            //开井
                                            openFlag = true;
                                        } else {
                                            //关井
                                            openFlag = false;
                                        }
                                        break;
                                    case 0:
                                        //相等为周期最后一天,只要关井天数不为0就是关井
                                        if (closeWellDay.equals(0)) {
                                            //开井
                                            openFlag = true;
                                        } else {
                                            //关井
                                            openFlag = false;
                                        }
                                        break;
                                    case 1:
                                        //当前未过一个执行周期,判断开井时间是否大于时差,大于就是开井
                                        if (openWellDay.compareTo(betweenDay) >= 0) {
                                            //开井
                                            openFlag = true;
                                        } else {
                                            //关井
                                            openFlag = false;
                                        }
                                        break;
                                    default:
                                }
                                String closeWellTime = BusinessConstant.START_OF_DAY_TIME;
                                if (openFlag) {
                                    closeWellTime = BusinessConstant.END_OF_DAY_TIME;
                                }
                                //创建间开记录
                                super.createDuration(unOptimizeDurationList, recordId, periodId, wellheadId, BusinessConstant.ONE, BusinessConstant.START_OF_DAY_TIME, closeWellTime, null);
                                super.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO, BusinessConstant.START_OF_DAY_TIME, closeWellTime, null);
                            }
                            break;
                        default:
                            //未配置不处理
                    }
                }
            }
            //需要优化的井口(匹配中短期优化配置)
            List<SpaceInstitutionWellheadEnt> needToOptimizeWellheadList = interval.get(true);
            if (CollUtil.isNotEmpty(needToOptimizeWellheadList)) {
                //查最近的中短期优化记录
                SpaceOptimizeMidPeriodMapper midPeriodMapper = context.getBean(SpaceOptimizeMidPeriodMapper.class);
                SpaceOptimizeMidPeriodEnt midPeriodEnt = midPeriodMapper.selectOne(new LambdaQueryWrapper<SpaceOptimizeMidPeriodEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceOptimizeMidPeriodEnt::getInstitutionId, detailId)
                        .orderByDesc(BaseModel::getCreateTime)
                        .last("LIMIT 1")
                );
                //如果有中短期优化记录,通过记录ID查优化过的时段按井口ID分组
                Map<String, List<SpaceOptimizeMidDurationEnt>> optimizeMidDurationMap = new HashMap<>(0);
                if (Objects.nonNull(midPeriodEnt)) {
                    SpaceOptimizeMidDurationMapper midDurationMapper = context.getBean(SpaceOptimizeMidDurationMapper.class);
                    List<SpaceOptimizeMidDurationEnt> optimizeMidDurationEntList = midDurationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeMidDurationEnt>()
                            .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                            .eq(SpaceOptimizeMidDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                            .eq(SpaceOptimizeMidDurationEnt::getMidPeriodId, midPeriodEnt.getId())
                            .orderByAsc(BaseModel::getCreateTime)
                    );
                    if (CollUtil.isNotEmpty(optimizeMidDurationEntList)) {
                        optimizeMidDurationMap = optimizeMidDurationEntList.stream()
                                .collect(Collectors.groupingBy(SpaceOptimizeMidDurationEnt::getWellheadId));
                    }
                }
                for (SpaceInstitutionWellheadEnt wellhead : needToOptimizeWellheadList) {
                    String wellheadId = wellhead.getWellheadId();
                    String recordId = super.createOptimizeWellhead(wellheadDTOList, periodId, wellheadId, wellhead.getWellNumber());
                    if (!durationListMap.containsKey(wellheadId)) {
                        continue;
                    }
                    List<SpaceInstitutionDurationEnt> durationList = durationListMap.get(wellheadId);
                    if (optimizeMidDurationMap.containsKey(wellheadId)) {
                        //优化记录转换
                        List<SpaceOptimizeMidDurationEnt> midDurationEntList = optimizeMidDurationMap.get(wellheadId);
                        for (SpaceOptimizeMidDurationEnt midDurationEnt : midDurationEntList) {
                            super.createDuration(durationDTOList, recordId, periodId, wellheadId, BusinessConstant.ZERO,
                                    midDurationEnt.getOpenWellTime(), midDurationEnt.getCloseWellTime(), midDurationEnt.getGenerationTypeKey());
                        }
                        //初始化未优化间开
                        for (SpaceInstitutionDurationEnt durationEnt : durationList) {
                            super.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId);
                        }
                    } else {
                        //缺少优化记录,不优化
                        for (SpaceInstitutionDurationEnt durationEnt : durationList) {
                            super.createUnOptimizeDuration(unOptimizeDurationList, durationEnt, periodId, recordId, wellheadId);
                            super.createOptimizeDuration(durationDTOList, periodId, recordId, wellheadId, null,
                                    durationEnt.getOpenWellTime(), durationEnt.getCloseWellTime()
                            );
                        }
                    }
                }
            }
        }
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(b -> optimizeViewMapper.shortPeriodBatchInsertList(b));
                } else {
                    optimizeViewMapper.shortPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(b -> optimizeViewMapper.shortWellheadBatchInsertList(b));
                } else {
                    optimizeViewMapper.shortWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(b -> optimizeViewMapper.shortDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.shortDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(b -> optimizeViewMapper.shortDurationBatchInsertList(b));
                } else {
                    optimizeViewMapper.shortDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
    }
}